package com.starbuds.server.api.notify.common.utils;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Duration;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

/**
 * 日期工具
 * @author aoe
 * @date 2020/7/1
 */
public class DateTools {

    private static final ZoneId ZONE_ID = ZoneId.systemDefault();

    private static final Locale LOCALE = Locale.CHINESE;

    public static Calendar toCalendar(String dateStr, SimpleDateFormat sdf) throws ParseException {
        Date date = sdf.parse(dateStr);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(date);

        return calendar;
    }

    public static Calendar nowCalendar(){
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        return calendar;
    }

    public static Instant toInstant(String dateStr, String pattern) {
        return LocalDateTime.parse(dateStr, getDateTimeFormatter(pattern))
                .atZone(ZONE_ID)
                .toInstant();
    }

    public static Instant nowInstant(){
        // 不要加时区，有的系统时区不生效
        return Instant.now();
    }

    public static SimpleDateFormat getSimpleDateFormat(String pattern) {
        return new SimpleDateFormat(pattern, LOCALE);
    }

    public static DateTimeFormatter getDateTimeFormatter(String pattern) {
        return DateTimeFormatter.ofPattern( pattern , LOCALE);
    }

    public static void main(String[] args) {
        Instant receiveTime = toInstant("2020-09-21 15:32:00", "yyyy-MM-dd HH:mm:ss");
        Instant now = nowInstant();
        long dif = Duration.between(receiveTime, now).toMillis();
        System.out.println(dif);
        System.out.println(Duration.between(receiveTime, now).toMinutes());
    }

}
