package com.starbuds.server.api.notify.v1.entity;

import lombok.Data;

import javax.ws.rs.FormParam;
import java.io.Serializable;

/**
 * 备注
 *
 * @author Lay
 * @date 2020/3/16
 */
@Data
public class HuaweiNotify implements Serializable {
    @FormParam("result")
    private String result;

    /**
     * 商户ID，即华为开发者联盟给开发者分配的“支付ID”。
     */
    @FormParam("userName")
    private String userName;

    /**
     * 商品名称
     */
    @FormParam("productName")
    private String productName;

    /**
     * 支付类型。取值范围如下：
     * ● 0：华为钱包
     * ● 3：信用卡
     * ● 4：支付宝
     * ● 6：短代
     * ● 13：PayPal
     * ● 16：借记卡
     * ● 17：微信
     * ● 19：礼品卡
     * ● 20：现金余额
     * ● 21：华为充值卡
     * ● 22：CyberSource
     * ● 24：WP
     * ● 31：HuaWeiWallet（华为Pay）
     * ● 32：花呗
     * ● 34：Adyen
     * ● 200：MPESA
     * ● 100-199：CUSTPAY1-100（支付定制支付方式）
     */
    @FormParam("payType")
    private Integer payType;

    /**
     * 商品支付金额，金额会保留2位小数，例如：20.00。
     */
    @FormParam("amount")
    private String amount;

    /**
     * 用于支付该商品的币种，必须符合ISO 4217标准，例如USD、CNY、MYR。
     * 如果支付时未传该参数或参数值为CNY，华为支付服务器回调时不传该参数。
     */
    @FormParam("currency")
    private String currency;

    /**
     * 华为支付平台订单号。
     */
    @FormParam("orderId")
    private String orderId;

    /**
     *
     * 时间戳，即自1970年1月1日0时起的毫秒数。
     * 通知时间为华为支付服务器调用接口的时间。
     */
    @FormParam("notifyTime")
    private String notifyTime;

    /**
     * 商户侧订单号。
     * 由开发者应用在支付时自行生成，用来唯一标识一次支付请求，订单号生成后开发者需自行维护支付订单号。
     */
    @FormParam("requestId")
    private String requestId;

    /**
     * 支付通道信息，通常为银行编码。该参数应用服务器可不关注。
     * 如下情况华为支付服务器会传入该参数：
     * ● 支付过程中，有具体的支付通道信息。
     * ● 应用在调用支付接口时urlVer参数配置项为“2”。
     */
    @FormParam("bankId")
    private String bankId;

    /**
     * 下单时间，即用户点击购买后华为支付服务器收到应用订单请求的时间，格式为yyyy-MM-dd hh:mm:ss。
     * 仅在应用调用支付接口时urlVer传入2时有效。
     */
    @FormParam("orderTime")
    private String orderTime;

    /**
     * 完成交易时间，格式为yyyy-MM-dd   hh:mm:ss。
     * 仅在应用调用支付接口时urlVer传入2时有效。
     */
    @FormParam("tradeTime")
    private String tradeTime;

    /**
     * 接入方式：
     * ● 0: 移动支付
     * ● 1: PC-Web
     * ● 2: Mobile-Web
     * ● 3: TV
     * 仅在应用调用支付接口时urlVer传入2时有效。
     */
    @FormParam("accessMode")
    private String accessMode;

    /**
     * 本参数无意义，仅为向前兼容保留。
     * 如果请求中携带了该参数且值不为null或""，需参与验签。
     */
    @FormParam("spending")
    private String spending;

    /**
     * 商户侧保留信息，华为支付服务器将传入支付时应用传入的extReserved信息。
     * 如果支付时未传入此参数，则华为支付服务器也不会传入该参数。
     */
    @FormParam("extReserved")
    private String extReserved;

    /**
     * 第三方支付商家侧保留信息。
     */
    @FormParam("sysReserved")
    private String sysReserved;

    /**
     * orderTime与tradeTime所在时区与零时区的时间差，单位ms，应用可根据此参数的值计算用户支付地的时区。参数默认值为28800000，表示+8区。-28800000则表示-8时区。
     * 时区换算方法：timeOffset/3600/1000。
     */
    @FormParam("timeOffset")
    private Long timeOffset;

    /**
     * 签名类型。
     *
     * ● 如果请求中携带该字段，取值默认为“RSA256”，表示华为支付服务器使用了SHA256WithRSA算法进行签名。
     * ● 如果请求中未携带该字段，表示华为支付服务器使用了SHA1WithRSA算法进行签名。
     */
    @FormParam("signType")
    private String signType;

    /**
     * 用于开发者服务器对支付结果通知消息进行验签的签名字符串，由华为支付服务器生成。
     * 开发者服务器需要参考对响应消息验签的方法对华为支付服务器发送的参数进行验签，验签通过后才能进行后续处理。验签使用的算法根据signType的取值确定。
     */
    @FormParam("sign")
    private String sign;
}
