package com.starbuds.server.api.notify.v1.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.notify.v1.entity.form.CaiyuCashoutNotifyForm;
import com.starbuds.server.api.notify.v1.facade.CaiyuFacade;
import com.starbuds.server.common.constant.finance.ThirdPartyCallbackEnum;
import com.starbuds.server.common.pojo.finance.CaiyuCashoutNotify;
import com.starbuds.server.common.pojo.finance.ThirdPartyCallbackHisSaveBO;
import com.starbuds.server.common.pojo.finance.bo.ThirdPartyCallbackBO;
import com.starbuds.server.common.service.finance.CaiyuService;
import com.starbuds.server.common.service.finance.ThirdPartyService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.jboss.resteasy.annotations.Form;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;

import static com.starbuds.server.api.notify.util.SpringContextUtil.isDev;

/**
 * @Author aoe
 * @DateTime 2021/01/19
 * @Description 财鱼
 */
@Path("/v1/caiyu")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class CaiyuFacadeImpl extends BaseFacade implements CaiyuFacade {

    /**
     * 处理成功
     */
    private final String SUCCESS = "ok";

    /**
     * 处理失败
     */
    private final String FAIL = "fail";

    @Reference(version = "1.0.0")
    private ThirdPartyService thirdPartyService;

    @Reference(version = "1.0.0")
    private CaiyuService caiyuService;

    @Context
    private HttpServletRequest request;

    @POST
    @Path("cashoutNotify")
    @PermitAll
    @Override
    public String cashoutNotify(@Form CaiyuCashoutNotifyForm form) {
        log.info("财鱼提现回调通知 = {}", form);
        CaiyuCashoutNotify notify = new CaiyuCashoutNotify();
        BeanUtils.copyProperties(form, notify);
        log.info("财鱼提现回调通知00 {}", JSON.toJSONString(notify, isDev()));

        if (notify == null || StringUtils.isBlank(notify.getResult()) || StringUtils.isBlank(notify.getPartnerOrderId())) {
            log.error("财鱼提现回调通知 参数不完整 {}", JSON.toJSONString(notify));
            return FAIL;
        }

        if (!caiyuService.checkSignCallback(notify)) {
            log.warn("财鱼提现回调通知 签名错误 {}", JSON.toJSONString(notify));
            return FAIL;
        }

        ThirdPartyCallbackHisSaveBO his = new ThirdPartyCallbackHisSaveBO();
        his.setInfoType(ThirdPartyCallbackEnum.CAIYU_CASHOUT_NOTIFY);
        his.setDetail(JSON.toJSONString(notify));
        his.setKeyword(String.format("%s,%s", notify.getResult(), notify.getPartnerOrderId()));
        boolean flag = thirdPartyService.saveCallbackHis(his);
        if (!flag) {
            log.error("财鱼提现回调通知 保存失败 {}", JSON.toJSONString(notify));
            return FAIL;
        }

        // 验证数据
        Long userWalletWithdrawHisId = NumberUtils.toLong(notify.getPartnerOrderId());
        if (userWalletWithdrawHisId < 1) {
            log.warn("财鱼提现回调通知 贵方订单号 partnerOrderId 有误 {}", notify.getPartnerOrderId());
            return FAIL;
        }
        String result = notify.getResult();
        if ("PAYING".equals(result)) {
            // 提现订单支付中，忽略处理
            return SUCCESS;
        }

        ThirdPartyCallbackBO callback = new ThirdPartyCallbackBO();
        callback.setUserWalletWithdrawHisId(userWalletWithdrawHisId);
        callback.setOutId(notify.getOrderId());
        callback.setIsSuccess("SUCCESS".equals(result));
        callback.setCallbackData(JSON.toJSONString(notify));
        callback.setCallbackMsg(notify.getPayMessage());

        return thirdPartyService.processWithdrawCallback(callback) ? SUCCESS : FAIL;
    }

    /**
     * 免实名电签回调通知
     *
     * @param body
     * @return
     */
    @POST
    @Path("signContractNoBankNotify")
    @Consumes("application/json; charset=utf-8")
    @PermitAll
    @Override
    public String signContractNoBankNotify(String body) {
        log.info("财鱼免实名电签回调通知 {}", body);
        if (StringUtils.isBlank(body)) {
            log.error("财鱼免实名电签回调通知 数据为空");
            return FAIL;
        }

        ThirdPartyCallbackHisSaveBO his = new ThirdPartyCallbackHisSaveBO();
        his.setInfoType(ThirdPartyCallbackEnum.CAIYU_SIGN_CONTRACT_NO_BANK_NOTIFY);
        his.setDetail(body);
        boolean flag = thirdPartyService.saveCallbackHis(his);
        if (!flag) {
            log.error("财鱼免实名电签回调通知 保存失败 {}", body);
            return FAIL;
        }

        return SUCCESS;
    }

}
