package com.starbuds.server.api.notify.v1.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.notify.util.RequestUtil;
import com.starbuds.server.api.notify.v1.facade.ChainFacade;
import com.starbuds.server.common.pojo.finance.ChainNotify;
import com.starbuds.server.common.service.finance.ChainService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;

/**
 * 链支付异步回调
 *
 * @author Lay
 * @date 2020/3/13
 */
@Path("/v1/chain")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class ChainFacadeImpl extends BaseFacade implements ChainFacade {

    @Reference(version = "1.0.0")
    ChainService chainService;

    /**
     * 成功
     */
    private static final String SUCCESS = "success";

    /**
     * 失败
     */
    private static final String FAIL = "fail";

    @POST
    @Path("tradeNotify")
    @Consumes(ContentType.APPLICATION_JSON_UTF_8)
    @PermitAll
    @Override
    public String tradeNotify(@Context HttpServletRequest request) {
        try {
            String notify = RequestUtil.httpServletRequestToString(request);
            log.info(notify);
            // 验签
            if (!chainService.checkNotifySign(notify)) {
                log.info("验签错误");
                return FAIL;
            }

            ChainNotify chainNotify = JSON.parseObject(notify, ChainNotify.class);
            if (chainNotify == null) {
                return FAIL;
            }

            // 通知处理
            if (chainService.handleNotify(chainNotify)) {
                return SUCCESS;
            }
        } catch (Exception e) {
            log.error("异步通知处理失败", e);
        }

        return FAIL;
    }
}
