package com.starbuds.server.api.notify.v1.impl;

import com.alibaba.fastjson.JSON;
import com.starbuds.server.api.notify.v1.facade.CloudFacade;
import com.starbuds.server.common.pojo.finance.CloudAccountOrderNotify;
import com.starbuds.server.common.service.finance.CloudAccountService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.ws.rs.FormParam;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/7/28 18:57
 * @Description 云账户
 */
@Path("/v1/cloudAccount")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class CloudFacadeImpl extends BaseFacade implements CloudFacade {

    public static final String SUCCESS = "success";
    public static final String FAIL = "fail";

    @Reference(version = "1.0.0")
    CloudAccountService cloudAccountService;

    @POST
    @Path("order")
    @PermitAll
    @Override
    public String order(@FormParam("data") String data,
                        @FormParam("mess") String mess,
                        @FormParam("sign") String sign,
                        @FormParam("sign_type") String signType,
                        @FormParam("timestamp") Long timestamp) {
        // 验证签名
        if (!"sha256".equals(signType)) {
            log.info("签名方式不一致");
            return FAIL;
        }
        if (!cloudAccountService.getSign(data, mess, timestamp).equals(sign)) {
            log.info("签名不一致");
            return FAIL;
        }
        // 解密data
        CloudAccountOrderNotify notify = JSON.parseObject(cloudAccountService.decodeData(data), CloudAccountOrderNotify.class);
        log.info("验签通过,notify:{}", JSON.toJSONString(notify));
        if (notify != null) {
            if (cloudAccountService.handleOrderNotify(notify)) {
                log.info("处理成功");
                return SUCCESS;
            }
            log.info("处理失败");
            return FAIL;
        }
        log.info("转换出错");
        return FAIL;
    }
}
