package com.starbuds.server.api.notify.v1.impl;

import com.starbuds.server.api.notify.v1.facade.DataFacade;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.BaiduNoticeDO;
import com.starbuds.server.common.service.data.DataService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/5/11 18:30
 * @Description 数据
 */
@Path("/v1/data")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class DataFacadeImpl extends BaseFacade implements DataFacade {

    @Reference(version = "1.0.0")
    DataService dataService;

    @GET
    @Path("baiduNotice")
    @PermitAll
    @Override
    public Result baiduNotice(@QueryParam("idfa") String idfa,
                              @QueryParam("imei_md5") String imeiMd5,
                              @QueryParam("os") String os,
                              @QueryParam("ip") String ip,
                              @QueryParam("ua") String ua,
                              @QueryParam("ts") Long ts,
                              @QueryParam("pid") String pid,
                              @QueryParam("uid") String uid,
                              @QueryParam("aid") String aid,
                              @QueryParam("click_id") String clickId,
                              @QueryParam("callback_url") String callbackUrl,
                              @QueryParam("sign") String sign) {

        if (StringUtils.isEmpty(idfa) && StringUtils.isEmpty(imeiMd5)) {
            return new Result(GlobalApiError.ServerError, "设备序列号不能为空");
        }

        BaiduNoticeDO noticeDO = new BaiduNoticeDO();
        noticeDO.setIdfa(idfa);
        noticeDO.setImeiMd5(imeiMd5);
        noticeDO.setOs(os);
        noticeDO.setIp(ip);
        noticeDO.setUa(ua);
        noticeDO.setTs(ts);
        noticeDO.setPid(pid);
        noticeDO.setUid(uid);
        noticeDO.setAid(aid);
        noticeDO.setClickId(clickId);
        noticeDO.setCallbackUrl(callbackUrl);
        noticeDO.setSign(sign);
        noticeDO.setCreateTime(System.currentTimeMillis());
        noticeDO.setStatus(CommonStatus.Normal);

        if (dataService.saveBaiduNotice(noticeDO)) {
            return new Result();
        }

        return new Result(GlobalApiError.ServerError);
    }

}
