package com.starbuds.server.api.notify.v1.impl;

import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.api.notify.util.RequestUtil;
import com.starbuds.server.api.notify.v1.facade.LiveFacade;
import com.starbuds.server.common.constant.agora.AgoraProductId;
import com.starbuds.server.common.constant.media.LiveStreamNotify;
import com.starbuds.server.common.constant.media.StreamStatusEnum;
import com.starbuds.server.common.constant.zego.RtcServiceCallback;
import com.starbuds.server.common.constant.zego.ZegoServerCallbackType;
import com.starbuds.server.common.pojo.rtc.RtcProviderSignature;
import com.starbuds.server.common.service.media.LiveService;
import com.starbuds.server.common.service.media.SingleChatService;
import com.starbuds.server.common.service.rtc.RtcSeatService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.FormParam;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;

/**
 * 直播通知接口实现
 *
 * @author Lay
 * @date 2019/12/23
 */
@Path("/v1/live")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class LiveFacadeImpl extends BaseFacade implements LiveFacade {

    private static final String SUCCESS = "success";
    private static final String FAILED = "failed";
    private static final String ROOMPREFIX = "YL-";
    private static final String ZEGO_EVENT_CREATE = "stream_create";
    private static final String ZEGO_EVENT_CLOSE = "stream_close";

    @Reference(version = "1.0.0")
    LiveService liveService;

    @Reference(version = "1.0.0")
    SingleChatService singleChatService;

    @Reference(version = "1.0.0")
    RtcSeatService rtcSeatService;

    @POST
    @Path("qiniuStreamStart")
    @PermitAll
    @Override
    public String qiniuStreamStart(@FormParam("userId") Long userId,
                                   @FormParam("ip") String ip,
                                   @FormParam("timestamp") Long timestamp,
                                   @FormParam("hid") Long liveHisId,
                                   @FormParam("sid") Long streamHisId) {
        LiveStreamNotify notify = LiveStreamNotify.builder().userId(userId).ip(ip).timesteamp(timestamp).liveHisId(liveHisId).streamHisId(streamHisId).streamStatus(StreamStatusEnum.Pushing).build();
        if (liveService.handleStreamNotify(notify)) {
            return SUCCESS;
        }
        return FAILED;
    }

    @POST
    @Path("qiniuStreamStop")
    @PermitAll
    @Override
    public String qiniuStreamStop(@FormParam("userId") Long userId,
                                  @FormParam("ip") String ip,
                                  @FormParam("timestamp") Long timestamp,
                                  @FormParam("hid") Long liveHisId,
                                  @FormParam("sid") Long streamHisId) {
        LiveStreamNotify notify = LiveStreamNotify.builder().userId(userId).ip(ip).timesteamp(timestamp).liveHisId(liveHisId).streamHisId(streamHisId).streamStatus(StreamStatusEnum.Closed).build();
        if (liveService.handleStreamNotify(notify)) {
            return SUCCESS;
        }
        return FAILED;
    }

    @POST
    @Path("agora")
    @PermitAll
    @Override
    public String agora(@Context HttpServletRequest request) {
        try {
            String notify = RequestUtil.httpServletRequestToString(request);
            String sign = request.getHeader("Agora-Signature");
            log.info(" RTC-agora request:" + notify);
            if (StringUtils.isBlank(notify) || StringUtils.isBlank(sign)) {
                return FAILED;
            }

            RtcProviderSignature signature = new RtcProviderSignature();
            signature.setNotify(notify);
            signature.setSignature(sign);
            if (!singleChatService.verifyAgoraNotifySign(signature)) {
                return FAILED;
            }

            JSONObject jsonNotify = JSONObject.parseObject(notify);
            if (jsonNotify == null) {
                return FAILED;
            }
            if (AgoraProductId.REAL_TIME != jsonNotify.getInteger("productId")) {
                return FAILED;
            }

            JSONObject payload = jsonNotify.getJSONObject("payload");
            Long timestamp = jsonNotify.getLong("notifyMs");
            Long eventType = jsonNotify.getLong("eventType");
            String channelName = payload.getString("channelName");
            String strUserID = payload.getString("uid");
            String strPlatform = payload.getString("platform");
            String strReason = payload.getString("reason");
            Long userID = StringUtils.isBlank(strUserID) ? 0L : Long.valueOf(strUserID);
            Integer platform = StringUtils.isBlank(strPlatform) ? 0 : Integer.valueOf(strPlatform);
            Integer reason = StringUtils.isBlank(strReason) ? 0 : Integer.valueOf(strReason);

            RtcServiceCallback callback = new RtcServiceCallback();
            callback.setChannelName(channelName);
            callback.setUserID(userID);
            callback.setTimestamp(timestamp);
            callback.setEventType(eventType.intValue());
            callback.setPlatform(platform);
            callback.setReason(reason);

            if (channelName.startsWith(ROOMPREFIX)) {
                return rtcSeatService.handleCallbackNotify(callback) ? SUCCESS : FAILED;
            }
            else {
                return singleChatService.handleCallbackNotify(callback) ? SUCCESS : FAILED;
            }
        } catch (Exception e) {
            log.error("声网异步通知处理失败", e);
        }
        return FAILED;
    }

    @POST
    @Path("zego")
    @PermitAll
    @Override
    public String zego(@Context HttpServletRequest request) {
        log.error("--zego---HttpServletRequest");
        try {
            RtcProviderSignature signature = new RtcProviderSignature();
            signature.setSignature(request.getParameter("signature"));
            signature.setTimestamp(Long.valueOf(request.getParameter("timestamp")));
            signature.setNonce(request.getParameter("nonce"));

            if (!singleChatService.verifyAgoraNotifySign(signature)) {
                return FAILED;
            }
            log.error("--zego---singleChatService.verifyAgoraNotifySign");
            JSONObject jsonObject = JSONObject.parseObject(RequestUtil.httpServletRequestToString(request));
            String event = jsonObject.getString("event");
            String channel = jsonObject.getString("room_id");
            String userID  = jsonObject.getString("user_id");
            Long timestamp = jsonObject.getLong("timestamp");

            log.error("\n\n--zego callback:" + event + " userId:" + userID);
            log.error("\n\n--zego HttpServletRequest:" + RequestUtil.httpServletRequestToString(request));

            RtcServiceCallback callback = new RtcServiceCallback();
            callback.setChannelName(channel);
            callback.setUserID(Long.valueOf(userID));
            callback.setTimestamp(timestamp);

            //默认情况下为关闭
            if (event.equals(ZEGO_EVENT_CREATE)) {
                callback.setEventType(ZegoServerCallbackType.StreamCreate);
            }
            else if (event.equals(ZEGO_EVENT_CLOSE)) {
                callback.setEventType(ZegoServerCallbackType.StreamClose);
                callback.setReason(jsonObject.getInteger("type"));
                callback.setPlatform(0);
            }
            else {
                return FAILED;
            }

            if (channel.startsWith(ROOMPREFIX)) {
                return rtcSeatService.handleCallbackNotify(callback) ? SUCCESS : FAILED;
            }
            else {
                return singleChatService.handleCallbackNotify(callback) ? SUCCESS : FAILED;
            }
        } catch (Exception e) {
            log.error("即构异步通知处理失败", e);
        }
        return FAILED;
    }
}
