package com.starbuds.server.api.notify.v1.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.api.notify.v1.entity.form.RongCloudAuditForm;
import com.starbuds.server.api.notify.v1.entity.form.RongCloudContent;
import com.starbuds.server.api.notify.v1.facade.RongCloudFacade;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.pojo.message.im.chat.ChatAuditFailedMsg;
import com.starbuds.server.common.service.message.ImService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;

/**
 * @author aoe
 * @date 2021/7/1
 */
@Path("/v1/rongCloud")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class RongCloudFacadeImpl extends BaseFacade implements RongCloudFacade {

    @Reference(version = "1.0.0")
    private ImService imService;

    @POST
    @Path("auditResult")
    @Consumes("application/json; charset=utf-8")
    @PermitAll
    @Override
    public String auditResult(String jsonStr) {
        log.info("融云聊天内容审核失败通知 {}", jsonStr);

        JSONObject body = JSON.parseObject(jsonStr);
        JSONObject content = JSON.parseObject(body.getString("content"));

        String msgId = body.getString("msgUID");
        if (StringUtils.isBlank(msgId)) {
            log.error("融云聊天内容审核失败通知 msgUID 为空");
            return "fail";
        }
        if (content == null) {
            log.error("融云聊天内容审核失败通知 content = null");
            return "fail";
        }
        String senderId = content.getString("fromUserId");
        if (StringUtils.isBlank(senderId)) {
            log.error("融云聊天内容审核失败通知 fromUserId 为空s");
            return "fail";
        }

        ChatAuditFailedMsg msg = new ChatAuditFailedMsg();
        msg.setMsgId(msgId);
        msg.setSenderId(NumberUtils.toLong(senderId));
        boolean flag = imService.sendCustomImMsg(null, msg.getSenderId(), msg, null);
        log.info("发送融云聊天内容审核失败消息 {} {}", JSON.toJSONString(msg, true), flag);

        return "ok";
    }
}
