package com.starbuds.server.api.notify.v1.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.starbuds.server.api.notify.v1.facade.YunxiaokeFacade;
import com.starbuds.server.common.constant.finance.ThirdPartyCallbackEnum;
import com.starbuds.server.common.constant.finance.ThirdPartyCompanyEnum;
import com.starbuds.server.common.pojo.finance.ThirdPartyCallbackHisDO;
import com.starbuds.server.common.pojo.finance.ThirdPartyCallbackHisSaveBO;
import com.starbuds.server.common.pojo.finance.ThirdPartyWithdrawHisDO;
import com.starbuds.server.common.pojo.finance.bo.ThirdPartyCallbackBO;
import com.starbuds.server.common.service.finance.ThirdPartyService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.security.PermitAll;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import java.nio.charset.Charset;

/**
 * 云销客异步回调
 * @author aoe
 * @date 2021/5/31
 */
@Path("/v1/yunxiaoke")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class YunxiaokeFacadeImpl extends BaseFacade implements YunxiaokeFacade {

    private static final String APPLICATION_OCTET_STREAM_UTF_8 = MediaType.APPLICATION_OCTET_STREAM + "; " + MediaType.CHARSET_PARAMETER + "=UTF-8";

    @Reference(version = "1.0.0")
    private ThirdPartyService thirdPartyService;


    /**
     * 转账回调
     * 返回的信息中需要进行：Unicode转中文
     * @param request
     * @return
     * @throws Exception
     */
    @POST
    @Path("ordersNotify")
    @Consumes(APPLICATION_OCTET_STREAM_UTF_8)
    @PermitAll
    @Override
    public JSONObject cashoutNotify(String request) {
        String signature = getHeaderString("signature");
        log.info("云销客异步回调 signature = {} ordersNotify {}", signature, request);

        JSONObject resErr = new JSONObject();
        resErr.put("success", "false");
        JSONObject body = JSON.parseObject(request);
        if (body == null) {
            log.error("云销客转账回调不是JSON格式 request = " + request);
            return resErr;
        }
        if (StringUtils.isBlank(signature)) {
            log.error("云销客转账回调 signature 为空 request = " + request);
            return resErr;
        }
        String signatureCheck = Base64.encodeBase64String(DigestUtils.getSha512Digest().digest(request.getBytes(Charset.forName("UTF-8"))));
        if (!signature.equals(signatureCheck)) {
            log.error(String.format("云销客转账回调签名错误 signature = %s signatureCheck = %s request = %s",
                    signature, signatureCheck, request));
            return resErr;
        }

        int orderStatus = body.getIntValue("order_status");
        String orderUuid = body.getString("order_uuid");

        ThirdPartyCallbackHisSaveBO his = new ThirdPartyCallbackHisSaveBO();
        his.setInfoType(ThirdPartyCallbackEnum.YUN_XIAO_KE_CASHOUT_NOTIFY);
        his.setDetail(request);
        his.setKeyword(String.format("%s,%s", orderStatus, orderUuid));
        boolean flag = thirdPartyService.saveCallbackHis(his);
        if (!flag) {
            log.error("云销客提现回调通知 保存失败 {}", request);
            return resErr;
        }

        // 验证数据
        ThirdPartyWithdrawHisDO po = thirdPartyService.getByOutId(orderUuid, ThirdPartyCompanyEnum.YUN_XIAO_KE);
        if (po == null) {
            log.warn("云销客提现回调通知 order_uuid 有误 {}，没有对应的提现记录", orderUuid);
            return resErr;
        }
        Long userWalletWithdrawHisId = po.getUserWalletWithdrawHisId();

        ThirdPartyCallbackBO callback = new ThirdPartyCallbackBO();
        callback.setUserWalletWithdrawHisId(userWalletWithdrawHisId);
        callback.setOutId(orderUuid);
        int successCode = 50;
        callback.setIsSuccess(successCode == orderStatus);
        callback.setCallbackData(request);
        callback.setCallbackMsg("-");

        if (!thirdPartyService.processWithdrawCallback(callback)) {
            log.error("云销客提现回调更新结果失败 request = " + request);
            return resErr;
        }
        log.info("云销客提现回调更新成功 callback = {}", callback);

        JSONObject res = new JSONObject();
        res.put("resp_code", "200");
        res.put("resp_message", "SUCCESS");
        res.put("success", "true");
        return res;
    }
}
